@@ -19,29 +19,99 @@ local rspamd_util = require "rspamd_util"
 local rspamd_regexp = require "rspamd_regexp"
 local tcp = require "rspamd_tcp"
 local upstream_list = require "rspamd_upstream_list"
+local lua_util = require "lua_util"
 local redis_params
 
 local N = "antivirus"
 
+if confighelp then
+  rspamd_config:add_example(nil, 'antivirus',
+    "Check messages for viruses",
+    [[
+antivirus {
+  # multiple scanners could be checked, for each we create a configuration block with an arbitrary name
+  clamav {
+    # If set force this action if any virus is found (default unset: no action is forced)
+    # action = "reject";
+    # if `true` only messages with non-image attachments will be checked (default true)
+    attachments_only = true;
+    # If `max_size` is set, messages > n bytes in size are not scanned
+    max_size = 20000000;
+    # symbol to add (add it to metric if you want non-zero weight)
+    symbol = "CLAM_VIRUS";
+    # type of scanner: "clamav", "fprot", "sophos" or "savapi"
+    type = "clamav";
+    # For "savapi" you must also specify the following variable
+    product_id = 12345;
+    # You can enable logging for clean messages
+    log_clean = true;
+    # servers to query (if port is unspecified, scanner-specific default is used)
+    # can be specified multiple times to pool servers
+    # can be set to a path to a unix socket
+    # Enable this in local.d/antivirus.conf
+    servers = "127.0.0.1:3310";
+    # if `patterns` is specified virus name will be matched against provided regexes and the related
+    # symbol will be yielded if a match is found. If no match is found, default symbol is yielded.
+    patterns {
+      # symbol_name = "pattern";
+      JUST_EICAR = "^Eicar-Test-Signature$";
+    }
+    # `whitelist` points to a map of IP addresses. Mail from these addresses is not scanned.
+    whitelist = "/etc/rspamd/antivirus.wl";
+  }
+}
+]])
+  return
+end
+
 local function match_patterns(default_sym, found, patterns)
-  if not patterns then return default_sym end
-  for sym, pat in pairs(patterns) do
-    if pat:match(found) then
-      return sym
+  if type(patterns) ~= 'table' then return default_sym end
+  if not patterns[1] then
+    for sym, pat in pairs(patterns) do
+      if pat:match(found) then
+        return sym
+      end
     end
+    return default_sym
+  else
+    for _, p in ipairs(patterns) do
+      for sym, pat in pairs(p) do
+        if pat:match(found) then
+          return sym
+        end
+      end
+    end
+    return default_sym
   end
-  return default_sym
 end
 
 local function yield_result(task, rule, vname)
-  local symname = match_patterns(rule['symbol'], vname, rule['patterns'])
-  if rule['whitelist'] and rule['whitelist']:get_key(vname) then
-    rspamd_logger.infox(task, '%s: "%s" is in whitelist', rule['type'], vname)
-    return
+  local all_whitelisted = true
+  if type(vname) == 'string' then
+    local symname = match_patterns(rule['symbol'], vname, rule['patterns'])
+    if rule['whitelist'] and rule['whitelist']:get_key(vname) then
+      rspamd_logger.infox(task, '%s: "%s" is in whitelist', rule['type'], vname)
+      return
+    end
+    task:insert_result(symname, 1.0, vname)
+    rspamd_logger.infox(task, '%s: virus found: "%s"', rule['type'], vname)
+  elseif type(vname) == 'table' then
+    for _, vn in ipairs(vname) do
+      local symname = match_patterns(rule['symbol'], vn, rule['patterns'])
+      if rule['whitelist'] and rule['whitelist']:get_key(vn) then
+        rspamd_logger.infox(task, '%s: "%s" is in whitelist', rule['type'], vn)
+      else
+        all_whitelisted = false
+        task:insert_result(symname, 1.0, vn)
+        rspamd_logger.infox(task, '%s: virus found: "%s"', rule['type'], vn)
+      end
+    end
   end
-  task:insert_result(symname, 1.0, vname)
-  rspamd_logger.infox(task, '%s: virus found: "%s"', rule['type'], vname)
   if rule['action'] then
+    if type(vname) == 'table' then
+      if all_whitelisted then return end
+      vname = table.concat(vname, '; ')
+    end
     task:set_pre_result(rule['action'],
         string.format('%s: virus found: "%s"', rule['type'], vname))
   end
@@ -199,7 +269,11 @@ end
 local function message_not_too_large(task, rule)
   local max_size = tonumber(rule['max_size'])
   if not max_size then return true end
-  if task:get_size() > max_size then return false end
+  if task:get_size() > max_size then
+    rspamd_logger.infox("skip %s AV check as it is too large: %s (%s is allowed)",
+      rule.type, task:get_size(), max_size)
+    return false
+  end
   return true
 end
 
@@ -211,6 +285,9 @@ local function need_av_check(task, rule)
       end
     end
 
+    rspamd_logger.infox("skip %s AV check as there are no attachments in a message",
+      rule.type)
+
     return false
   else
     return message_not_too_large(task, rule)
@@ -225,6 +302,7 @@ local function check_av_cache(task, rule, fn)
       -- Cached
       if data ~= 'OK' then
         rspamd_logger.debugm(N, task, 'got cached result for %s: %s', key, data)
+        data = rspamd_str_split(data, '\x30')
         yield_result(task, rule, data)
       else
         rspamd_logger.debugm(N, task, 'got cached result for %s: %s', key, data)
@@ -269,6 +347,10 @@ local function save_av_cache(task, rule, to_save)
     end
   end
 
+  if type(to_save) == 'table' then
+    to_save = table.concat(to_save, '\x30')
+  end
+
   if redis_params then
     key = rule['prefix'] .. key
 
@@ -328,7 +410,9 @@ local function fprot_check(task, rule)
             rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
           end
         else
-          local vname = string.match(data, '^1 <infected: (.+)>')
+          -- returncodes: 1: infected, 2: suspicious, 3: both, 4-255: some error occured
+          -- see http://www.f-prot.com/support/helpfiles/unix/appendix_c.html for more detail
+          local vname = string.match(data, '^[1-3] <[%w%s]-: (.-)>')
           if not vname then
             rspamd_logger.errx(task, 'Unhandled response: %s', data)
           else
@@ -446,7 +530,7 @@ local function sophos_check(task, rule)
     local streamsize = string.format('SCANDATA %d\n', task:get_size())
     local bye = 'BYE\n'
 
-    local function sophos_callback(err, data)
+    local function sophos_callback(err, data, conn)
       if err then
         if err == 'IO timeout' then
           if retransmits > 0 then
@@ -480,6 +564,8 @@ local function sophos_check(task, rule)
               rspamd_logger.infox(task, '%s [%s]: message is clean', rule['symbol'], rule['type'])
             end
             save_av_cache(task, rule, 'OK')
+          elseif string.find(data, 'ACC') or string.find(data, 'OK SSSP') then
+            conn:add_read(sophos_callback)
           else
             rspamd_logger.errx(task, 'unhandled response: %s', data)
           end
@@ -512,8 +598,27 @@ local function savapi_check(task, rule)
     local addr = upstream:get_addr()
     local retransmits = rule.retransmits
     local message_file = task:store_in_file(tonumber("0644", 8))
+    local vnames = {}
+
+    -- Forward declaration for recursive calls
+    local savapi_scan1_cb
 
     local function savapi_fin_cb(err, conn)
+      local vnames_reordered = {}
+      -- Swap table
+      for virus,_ in pairs(vnames) do
+        table.insert(vnames_reordered, virus)
+      end
+      rspamd_logger.debugm(N, task, "%s: number of virus names found %s", rule['type'], #vnames_reordered)
+      if #vnames_reordered > 0 then
+        local vname = {}
+        for _,virus in ipairs(vnames_reordered) do
+          table.insert(vname, virus)
+        end
+
+        yield_result(task, rule, vname)
+        save_av_cache(task, rule, vname)
+      end
       if conn then
         conn:close()
       end
@@ -523,24 +628,37 @@ local function savapi_check(task, rule)
       local result = tostring(data)
       rspamd_logger.debugm(N, task, "%s: got reply: %s", rule['type'], result)
 
+      -- Terminal response - clean
       if string.find(result, '200') or string.find(result, '210') then
         if rule['log_clean'] then
           rspamd_logger.infox(task, '%s: message is clean', rule['type'])
         end
         save_av_cache(task, rule, 'OK')
+        conn:add_write(savapi_fin_cb, 'QUIT\n')
 
+      -- Terminal response - infected
+      elseif string.find(result, '319') then
+        conn:add_write(savapi_fin_cb, 'QUIT\n')
+
+      -- Non-terminal response
       elseif string.find(result, '310') then
-        -- Recursive result
-        local parts = rspamd_str_split(result, ' <<< ')
-        local vname = rspamd_str_split(parts[2], ';')[1]:match "^%s*(.-)%s*$"
-        rspamd_logger.infox(task, '%s: virus found: %s', rule['type'], vname)
-        yield_result(task, rule, vname)
-        save_av_cache(task, rule, vname)
+        local virus
+        virus = result:match "310.*<<<%s(.*)%s+;.*;.*"
+        if not virus then
+          virus = result:match "310%s(.*)%s+;.*;.*"
+          if not virus then
+            rspamd_logger.errx(task, "%s: virus result unparseable: %s", rule['type'], result)
+            return
+          end
+        end
+        -- Store unique virus names
+        vnames[virus] = 1
+        -- More content is expected
+        conn:add_write(savapi_scan1_cb, '\n')
       end
-      conn:add_write(savapi_fin_cb, 'QUIT\n')
     end
 
-    local function savapi_scan1_cb(err, conn)
+    savapi_scan1_cb = function(err, conn)
       conn:add_read(savapi_scan2_cb, '\n')
     end
 
@@ -631,7 +749,6 @@ local av_types = {
 }
 
 local function add_antivirus_rule(sym, opts)
-  local rule = {}
   if not opts['type'] then
     return nil
   end
@@ -644,7 +761,9 @@ local function add_antivirus_rule(sym, opts)
       opts['type'])
   end
 
-  rule = cfg.configure(opts)
+  local rule = cfg.configure(opts)
+  rule.type = opts.type
+
 
   if not rule then
     rspamd_logger.errx(rspamd_config, 'cannot configure %s for %s',
@@ -652,10 +771,24 @@ local function add_antivirus_rule(sym, opts)
     return nil
   end
 
-  if opts['patterns'] then
+  if type(opts['patterns']) == 'table' then
     rule['patterns'] = {}
-    for k, v in pairs(opts['patterns']) do
-      rule['patterns'][k] = rspamd_regexp.create_cached(v)
+    if opts['patterns'][1] then
+      for i, p in ipairs(opts['patterns']) do
+        if type(p) == 'table' then
+          local new_set = {}
+          for k, v in pairs(p) do
+            new_set[k] = rspamd_regexp.create_cached(v)
+          end
+          rule['patterns'][i] = new_set
+        else
+          rule['patterns'][i] = {}
+        end
+      end
+    else
+      for k, v in pairs(opts['patterns']) do
+        rule['patterns'][k] = rspamd_regexp.create_cached(v)
+      end
     end
   end
 
@@ -672,6 +805,7 @@ end
 local opts = rspamd_config:get_all_opt('antivirus')
 if opts and type(opts) == 'table' then
   redis_params = rspamd_parse_redis_server('antivirus')
+  local has_valid = false
   for k, m in pairs(opts) do
     if type(m) == 'table' and m['type'] and m['servers'] then
       local cb = add_antivirus_rule(k, m)
@@ -683,13 +817,28 @@ if opts and type(opts) == 'table' then
           name = m['symbol'],
           callback = cb,
         })
-        if m['patterns'] then
-          for sym in pairs(m['patterns']) do
-            rspamd_config:register_symbol({
-              type = 'virtual',
-              name = sym,
-              parent = id
-            })
+        has_valid = true
+        if type(m['patterns']) == 'table' then
+          if m['patterns'][1] then
+            for _, p in ipairs(m['patterns']) do
+              if type(p) == 'table' then
+                for sym in pairs(p) do
+                  rspamd_config:register_symbol({
+                    type = 'virtual',
+                    name = sym,
+                    parent = id
+                  })
+                end
+              end
+            end
+          else
+            for sym in pairs(m['patterns']) do
+              rspamd_config:register_symbol({
+                type = 'virtual',
+                name = sym,
+                parent = id
+              })
+            end
           end
         end
         if m['score'] then
@@ -712,4 +861,9 @@ if opts and type(opts) == 'table' then
       end
     end
   end
+
+  if not has_valid then
+    lua_util.disable_module(N, 'config')
+  end
 end
+
