@@ -59,7 +59,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = 'No received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_ONE',
@@ -67,7 +67,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = 'One received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_TWO',
@@ -83,7 +83,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '3-5 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_FIVE',
@@ -91,7 +91,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '5-7 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_SEVEN',
@@ -99,7 +99,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '7-11 received',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCVD_COUNT_TWELVE',
@@ -107,7 +107,7 @@ rspamd_config:register_symbol{
   parent = rcvd_cb_id,
   type = 'virtual',
   description = '12+ received',
-  group = 'header',
+  group = 'headers',
 }
 
 local prio_cb_id = rspamd_config:register_symbol {
@@ -141,7 +141,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 0',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_ONE',
@@ -149,7 +149,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 1',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_TWO',
@@ -157,7 +157,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 2',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_THREE',
@@ -165,7 +165,7 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 3-4',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_X_PRIO_FIVE',
@@ -173,36 +173,38 @@ rspamd_config:register_symbol{
   parent = prio_cb_id,
   type = 'virtual',
   description = 'Priority 5+',
-  group = 'header',
+  group = 'headers',
 }
 
+local function get_raw_header(task, name)
+  return ((task:get_header_full(name) or {})[1] or {})['value']
+end
+
 local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO', 1.0,
   function (task)
-    local replyto = task:get_header('Reply-To')
+    local replyto = get_raw_header(task, 'Reply-To')
     if not replyto then return false end
     local rt = util.parse_mail_address(replyto)
-    if not (rt and rt[1]) then
+    if not (rt and rt[1] and (string.len(rt[1].addr) > 0)) then
       task:insert_result('REPLYTO_UNPARSEABLE', 1.0)
       return false
     else
-      task:insert_result('HAS_REPLYTO', 1.0)
       local rta = rt[1].addr
-      if rta then
-        -- Check if Reply-To address starts with title seen in display name
-        local sym = task:get_symbol('FROM_NAME_HAS_TITLE')
-        local title = (((sym or E)[1] or E).options or E)[1]
-        if title then
-          rta = rta:lower()
-          if rta:find('^' .. title) then
-            task:insert_result('REPLYTO_EMAIL_HAS_TITLE', 1.0)
-          end
+      task:insert_result('HAS_REPLYTO', 1.0, rta)
+      -- Check if Reply-To address starts with title seen in display name
+      local sym = task:get_symbol('FROM_NAME_HAS_TITLE')
+      local title = (((sym or E)[1] or E).options or E)[1]
+      if title then
+        rta = rta:lower()
+        if rta:find('^' .. title) then
+          task:insert_result('REPLYTO_EMAIL_HAS_TITLE', 1.0)
         end
       end
     end
 
     -- See if Reply-To matches From in some way
     local from = task:get_from(2)
-    local from_h = task:get_header('From')
+    local from_h = get_raw_header(task, 'From')
     if not (from and from[1]) then return false end
     if (from_h and from_h == replyto) then
       -- From and Reply-To are identical
@@ -210,17 +212,30 @@ local check_replyto_id = rspamd_config:register_callback_symbol('CHECK_REPLYTO',
     else
       if (from and from[1]) then
         -- See if From and Reply-To addresses match
-        if (from[1].addr:lower() == rt[1].addr:lower()) then
+        if (util.strequal_caseless(from[1].addr, rt[1].addr)) then
           task:insert_result('REPLYTO_ADDR_EQ_FROM', 1.0)
         elseif from[1].domain and rt[1].domain then
-          if (from[1].domain:lower() == rt[1].domain:lower()) then
+          if (util.strequal_caseless(from[1].domain, rt[1].domain)) then
             task:insert_result('REPLYTO_DOM_EQ_FROM_DOM', 1.0)
-          else
-            task:insert_result('REPLYTO_DOM_NEQ_FROM_DOM', 1.0)
+          else 
+            -- See if Reply-To matches the To address
+            local to = task:get_recipients(2)
+            if (to and to[1] and to[1].addr:lower() == rt[1].addr:lower()) then
+                -- Ignore this for mailing-lists and automatic submissions
+                if (not (task:get_header('List-Unsubscribe') or
+                         task:get_header('X-List') or
+                         task:get_header('Auto-Submitted'))) 
+                then
+                   task:insert_result('REPLYTO_EQ_TO_ADDR', 1.0)
+                end
+            else
+              task:insert_result('REPLYTO_DOM_NEQ_FROM_DOM', 1.0)
+	    end
           end
         end
         -- See if the Display Names match
-        if (from[1].name and rt[1].name and from[1].name:lower() == rt[1].name:lower()) then
+        if (from[1].name and rt[1].name and
+            util.strequal_caseless(from[1].name, rt[1].name)) then
           task:insert_result('REPLYTO_DN_EQ_FROM_DN', 1.0)
         end
       end
@@ -234,7 +249,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header could not be parsed',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'HAS_REPLYTO',
@@ -242,7 +257,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Has Reply-To header',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_EQ_FROM',
@@ -250,7 +265,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header is identical to From header',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_ADDR_EQ_FROM',
@@ -258,7 +273,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header is identical to SMTP From',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DOM_EQ_FROM_DOM',
@@ -266,7 +281,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To domain matches the From domain',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DOM_NEQ_FROM_DOM',
@@ -274,7 +289,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To domain does not match the From domain',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_DN_EQ_FROM_DN',
@@ -282,7 +297,7 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To display name matches From',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'REPLYTO_EMAIL_HAS_TITLE',
@@ -290,9 +305,18 @@ rspamd_config:register_symbol{
   parent = check_replyto_id,
   type = 'virtual',
   description = 'Reply-To header has title',
-  group = 'header',
+  group = 'headers',
 }
-rspamd_config:register_dependency(check_replyto_id, 'FROM_NAME_HAS_TITLE')
+rspamd_config:register_symbol{
+  name = 'REPLYTO_EQ_TO_ADDR',
+  score = 5.0,
+  parent = check_replyto_id,
+  type = 'virtual',
+  description = 'Reply-To is the same as the To address',
+  group = 'headers',
+}
+
+rspamd_config:register_dependency('CHECK_REPLYTO', 'CHECK_FROM')
 
 local check_mime_id = rspamd_config:register_symbol{
   name = 'CHECK_MIME',
@@ -356,7 +380,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME-Version header is missing',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'MIME_MA_MISSING_TEXT',
@@ -364,7 +388,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME multipart/alternative missing text/plain part',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'MIME_MA_MISSING_HTML',
@@ -372,7 +396,7 @@ rspamd_config:register_symbol{
   parent = check_mime_id,
   type = 'virtual',
   description = 'MIME multipart/alternative missing text/html part',
-  group = 'header',
+  group = 'headers',
 }
 
 -- Used to be called IS_LIST
@@ -402,7 +426,7 @@ rspamd_config.BROKEN_HEADERS = {
     return task:has_flag('broken_headers')
   end,
   score = 10.0,
-  group = 'header',
+  group = 'headers',
   description = 'Headers structure is likely broken'
 }
 
@@ -412,7 +436,7 @@ rspamd_config.BROKEN_CONTENT_TYPE = {
       task:get_parts())
   end,
   score = 1.5,
-  group = 'header',
+  group = 'headers',
   description = 'Message has part with broken content type'
 }
 
@@ -441,7 +465,7 @@ rspamd_config.HEADER_RCONFIRM_MISMATCH = {
   end,
 
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Read confirmation address is different to from address'
 }
 
@@ -474,7 +498,7 @@ rspamd_config.HEADER_FORGED_MDN = {
   end,
 
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Read confirmation address is different to return path'
 }
 
@@ -517,7 +541,8 @@ rspamd_config.MULTIPLE_UNIQUE_HEADERS = {
   end,
 
   score = 5.0,
-  group = 'header',
+  group = 'headers',
+  one_shot = true,
   description = 'Repeated unique headers'
 }
 
@@ -530,7 +555,7 @@ rspamd_config.MISSING_FROM = {
     return false
   end,
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   description = 'Missing From: header'
 }
 rspamd_config.MV_CASE = {
@@ -539,7 +564,8 @@ rspamd_config.MV_CASE = {
     if (mv) then return true end
   end,
   description = 'Mime-Version .vs. MIME-Version',
-  score = 0.5
+  score = 0.5,
+  group = 'headers',
 }
 
 rspamd_config.FAKE_REPLY = {
@@ -553,7 +579,8 @@ rspamd_config.FAKE_REPLY = {
     return false
   end,
   description = 'Fake reply',
-  score = 1.0
+  score = 1.0,
+  group = 'headers'
 }
 
 local check_from_id = rspamd_config:register_symbol{
@@ -562,12 +589,12 @@ local check_from_id = rspamd_config:register_symbol{
   callback = function(task)
     local envfrom = task:get_from(1)
     local from = task:get_from(2)
-    if (from and from[1] and not from[1].name) then
+    if (from and from[1] and (from[1].name == nil or from[1].name == '' )) then
       task:insert_result('FROM_NO_DN', 1.0)
     elseif (from and from[1] and from[1].name and
-            from[1].name:lower() == from[1].addr:lower()) then
+            util.strequal_caseless(from[1].name, from[1].addr)) then
       task:insert_result('FROM_DN_EQ_ADDR', 1.0)
-    elseif (from and from[1] and from[1].name) then
+    elseif (from and from[1] and from[1].name and from[1].name ~= '') then
       task:insert_result('FROM_HAS_DN', 1.0)
       -- Look for Mr/Mrs/Dr titles
       local n = from[1].name:lower()
@@ -586,20 +613,21 @@ local check_from_id = rspamd_config:register_symbol{
       end
     end
     if (envfrom and from and envfrom[1] and from[1] and
-        envfrom[1].addr:lower() == from[1].addr:lower())
+        util.strequal_caseless(envfrom[1].addr, from[1].addr))
     then
       task:insert_result('FROM_EQ_ENVFROM', 1.0)
     elseif (envfrom and envfrom[1] and envfrom[1].addr) then
-      task:insert_result('FROM_NEQ_ENVFROM', 1.0, from and from[1].addr or '', envfrom[1].addr)
+      task:insert_result('FROM_NEQ_ENVFROM', 1.0, ((from or E)[1] or E).addr or '', envfrom[1].addr)
     end
 
     local to = task:get_recipients(2)
-    if not (to and to[1] and #to == 1 and from) then return false end
+    if not (to and to[1] and #to == 1 and from and from[1]) then return false end
     -- Check if FROM == TO
-    if (to[1].addr:lower() == from[1].addr:lower()) then
+    if (util.strequal_caseless(to[1].addr, from[1].addr)) then
       task:insert_result('TO_EQ_FROM', 1.0)
     elseif (to[1].domain and from[1].domain and
-        to[1].domain:lower() == from[1].domain:lower()) then
+            util.strequal_caseless(to[1].domain, from[1].domain))
+    then
       task:insert_result('TO_DOM_EQ_FROM_DOM', 1.0)
     end
   end
@@ -608,7 +636,7 @@ local check_from_id = rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NO_DN',
   score = 0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header does not have a display name',
@@ -616,7 +644,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_DN_EQ_ADDR',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name is the same as the address',
@@ -624,7 +652,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_HAS_DN',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header has a display name',
@@ -632,7 +660,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NAME_EXCESS_SPACE',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name contains excess whitespace',
@@ -640,7 +668,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NAME_HAS_TITLE',
   score = 1.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From header display name has a title (Mr/Mrs/Dr)',
@@ -648,7 +676,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_EQ_ENVFROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From address is the same as the envelope',
@@ -656,7 +684,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'FROM_NEQ_ENVFROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'From address is different to the envelope',
@@ -664,7 +692,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_EQ_FROM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'To address matches the From address',
@@ -672,7 +700,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DOM_EQ_FROM_DOM',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_from_id,
   type = 'virtual',
   description = 'To domain is the same as the From domain',
@@ -713,16 +741,16 @@ local check_to_cc_id = rspamd_config:register_symbol{
           or toa['name']:lower() == 'recipients')) then
         task:insert_result('TO_DN_RECIPIENTS', 1.0)
       end
-      if (toa['name'] and toa['name']:lower() == toa['addr']:lower()) then
+      if (toa['name'] and util.strequal_caseless(toa['name'], toa['addr'])) then
         to_dn_eq_addr_count = to_dn_eq_addr_count + 1
-      elseif (toa['name']) then
+      elseif (toa['name'] and toa['name'] ~= '') then
         to_dn_count = to_dn_count + 1
       end
       -- See if header recipients match envrcpts
       if (rcpts) then
         for _, rcpt in ipairs(rcpts) do
           if (toa and toa['addr'] and rcpt and rcpt['addr'] and
-              rcpt['addr']:lower() == toa['addr']:lower())
+              util.strequal_caseless(rcpt['addr'], toa['addr']))
           then
             to_match_envrcpt = to_match_envrcpt + 1
           end
@@ -757,7 +785,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'No recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_ONE',
@@ -765,7 +793,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'One recipient',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_TWO',
@@ -773,7 +801,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Two recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_THREE',
@@ -781,7 +809,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '3-5 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_FIVE',
@@ -789,7 +817,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '5-7 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_SEVEN',
@@ -797,7 +825,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '7-11 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_TWELVE',
@@ -805,7 +833,7 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '12-50 recipients',
-  group = 'header',
+  group = 'headers',
 }
 rspamd_config:register_symbol{
   name = 'RCPT_COUNT_GT_50',
@@ -813,13 +841,13 @@ rspamd_config:register_symbol{
   parent = check_to_cc_id,
   type = 'virtual',
   description = '50+ recipients',
-  group = 'header',
+  group = 'headers',
 }
 
 rspamd_config:register_symbol{
   name = 'TO_DN_RECIPIENTS',
   score = 2.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'To header display name is "Recipients"',
@@ -827,7 +855,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_NONE',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'None of the recipients have display names',
@@ -835,7 +863,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All the recipients have display names',
@@ -843,7 +871,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients have display names',
@@ -851,7 +879,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_EQ_ADDR_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All of the recipients have display names that are the same as their address',
@@ -859,7 +887,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_DN_EQ_ADDR_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients have display names that are the same as their address',
@@ -867,7 +895,7 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_MATCH_ENVRCPT_ALL',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'All of the recipients match the envelope',
@@ -875,8 +903,143 @@ rspamd_config:register_symbol{
 rspamd_config:register_symbol{
   name = 'TO_MATCH_ENVRCPT_SOME',
   score = 0.0,
-  group = 'header',
+  group = 'headers',
   parent = check_to_cc_id,
   type = 'virtual',
   description = 'Some of the recipients match the envelope',
 }
+
+rspamd_config.CTYPE_MISSING_DISPOSITION = {
+  callback = function(task)
+    local parts = task:get_parts()
+    if (not parts) or (parts and #parts < 1) then return false end
+    for _,p in ipairs(parts) do
+      local ct = p:get_header('Content-Type')
+      if (ct and ct:lower():match('^application/octet%-stream') ~= nil) then
+        local cd = p:get_header('Content-Disposition')
+        if (not cd) or (cd and cd:lower():find('^attachment') == nil) then
+          local ci = p:get_header('Content-ID')
+          if ci or (#parts > 1 and (cd and cd:find('filename=.+%.asc') ~= nil))
+          then
+            return false
+          end
+          return true
+        end
+      end
+    end
+    return false
+  end,
+  description = 'Binary content-type not specified as an attachment',
+  score = 4.0,
+  group = 'headers'
+}
+
+rspamd_config.CTYPE_MIXED_BOGUS = {
+  callback = function(task)
+    local ct = task:get_header('Content-Type')
+    if (not ct) then return false end
+    local parts = task:get_parts()
+    if (not parts) then return false end
+    if (not ct:lower():match('^multipart/mixed')) then return false end
+    local found = false
+    -- Check each part and look for a part that isn't multipart/* or text/plain or text/html
+    for _,p in ipairs(parts) do
+      local pct = p:get_header('Content-Type')
+      if (pct) then
+        pct = pct:lower()
+        if not ((pct:match('^multipart/') or
+            pct:match('^text/plain') or
+            pct:match('^text/html'))) then
+          found = true
+        end
+      end
+    end
+    if (not found) then return true end
+    return false
+  end,
+  description = 'multipart/mixed without non-textual part',
+  score = 1.0,
+  group = 'headers'
+}
+
+local function check_for_base64_text(part)
+  local ct = part:get_header('Content-Type')
+  if (not ct) then return false end
+  ct = ct:lower()
+  if (ct:match('^text')) then
+    -- Check encoding
+    local cte = part:get_header('Content-Transfer-Encoding')
+    if (cte and cte:lower():match('^base64')) then
+      return true
+    end
+  end
+  return false
+end
+
+rspamd_config.MIME_BASE64_TEXT = {
+  callback = function(task)
+    -- Check outer part
+    if (check_for_base64_text(task)) then
+      return true
+    else
+      local parts = task:get_parts()
+      if (not parts) then return false end
+      -- Check each part and look for base64 encoded text parts
+      for _, part in ipairs(parts) do
+        if (check_for_base64_text(part)) then
+          return true
+        end
+      end
+    end
+    return false
+  end,
+  description = 'Has text part encoded in base64',
+  score = 0.1,
+  group = 'headers'
+}
+
+local function is_8bit_addr(addr)
+  if addr.flags and addr.flags['8bit'] then
+    return true
+  end
+
+  return false;
+end
+
+rspamd_config.INVALID_FROM_8BIT = {
+  callback = function(task)
+    local from = (task:get_from('mime') or {})[1] or {}
+    if is_8bit_addr(from) then
+      return true
+    end
+    return false
+  end,
+  description = 'Invalid 8bit character in From header',
+  score = 6.0,
+  group = 'headers'
+}
+
+rspamd_config.INVALID_RCPT_8BIT = {
+  callback = function(task)
+    local rcpts = task:get_recipients('mime') or {}
+    return fun.any(function(rcpt)
+      if is_8bit_addr(rcpt) then
+        return true
+      end
+      return false
+    end, rcpts)
+  end,
+  description = 'Invalid 8bit character in recipients headers',
+  score = 6.0,
+  group = 'headers'
+}
+
+rspamd_config.XM_CASE = {
+  callback = function (task)
+    local xm = task:get_header('X-mailer', true)
+    if (xm) then return true end
+  end,
+  description = 'X-mailer .vs. X-Mailer',
+  score = 0.5,
+  group = 'headers',
+}
