@@ -28,8 +28,8 @@ reconf['PRECEDENCE_BULK'] = {
 reconf['MICROSOFT_SPAM'] = {
     -- https://technet.microsoft.com/en-us/library/dn205071(v=exchg.150).aspx
     re = 'X-Forefront-Antispam-Report=/SFV:SPM/H',
-    score = 10,
-    description = "Microsoft says the messge is spam",
+    score = 4,
+    description = "Microsoft says the message is spam",
     group = 'upstream_spam_filters'
 }
 
@@ -41,8 +41,15 @@ reconf['AOL_SPAM'] = {
 }
 
 reconf['SPAM_FLAG'] = {
-    re = 'X-Spam-Flag=/^(?:yes|true)/Hi',
+    re = string.format('%s || %s', 'X-Spam-Flag=/^(?:yes|true)/Hi', 'X-Spam=/^yes$/Hi'),
     score = 5,
     description = "Message was already marked as spam",
     group = 'upstream_spam_filters'
 }
+
+reconf['UNITEDINTERNET_SPAM'] = {
+    re = 'X-UI-Out-Filterresults=/^junk:/H',
+    score = 5,
+    description = "United Internet says this message is spam",
+    group = 'upstream_spam_filters'
+}
