@@ -23,7 +23,7 @@
  THE SOFTWARE.
  */
 
-define(['jquery', 'd3evolution', 'datatables'],
+define(['jquery', 'd3evolution', 'footable'],
 function($, D3Evolution, unused) {
     var rrd_pie_config = {
         header: {},
@@ -64,7 +64,7 @@ function($, D3Evolution, unused) {
                 label: "Rejected",
                 color: "#FF0000"
             }, {
-                label: "Temporary rejected",
+                label: "Temporarily rejected",
                 color: "#BF8040"
             }, {
                 label: "Subject rewritten",
@@ -90,10 +90,14 @@ function($, D3Evolution, unused) {
 
     function initGraph() {
         var graph = new D3Evolution("graph", $.extend({}, graph_options, {
+            yScale:      getSelector("selYScale"),
             type:        getSelector("selType"),
             interpolate: getSelector("selInterpolate"),
             convert:     getSelector("selConvert"),
         }));
+        $("#selYScale").change(function() {
+            graph.yScale(this.value);
+        });
         $("#selConvert").change(function () {
             graph.convert(this.value);
         });
@@ -112,40 +116,49 @@ function($, D3Evolution, unused) {
         var timeInterval = xExtents[1] - xExtents[0];
 
         return json.map(function (curr, i) {
+            // Time intervals that don't have data are excluded from average calculation as d3.mean()ignores nulls
             var avg = d3.mean(curr, function (d) { return d.y; });
+            // To find an integral on the whole time interval we need to convert nulls to zeroes
+            var value = d3.mean(curr, function (d) { return +d.y; }) * timeInterval / scaleFactor;
             var yExtents = d3.extent(curr, function (d) { return d.y; });
 
             return {
                 label: graph_options.legend.entries[i].label,
-                value: avg && (avg * timeInterval / scaleFactor) ^ 0,
-                min: yExtents[0],
-                avg: avg && avg.toFixed(6),
-                max: yExtents[1],
-                last: curr[curr.length - 1].y,
+                value: value ^ 0,
+                min: +yExtents[0].toFixed(6),
+                avg: +avg.toFixed(6),
+                max: +yExtents[1].toFixed(6),
+                last: +curr[curr.length - 1].y.toFixed(6),
                 color: graph_options.legend.entries[i].color,
             };
         }, []);
     }
 
     function drawRrdTable(data, unit) {
-        $('#rrd-table').DataTable({
-            destroy: true,
-            paging: false,
-            searching: false,
-            info: false,
-            data: data,
-            columns: [
-                { data: "label", title: "Action" },
-                { data: "value", title: "Messages",       defaultContent: "" },
-                { data: "min",   title: "Minimum, " + unit, defaultContent: "" },
-                { data: "avg",   title: "Average, " + unit, defaultContent: "" },
-                { data: "max",   title: "Maximum, " + unit, defaultContent: "" },
-                { data: "last",  title: "Last, " + unit },
-            ],
+        var rows = data.map(function (curr, i) {
+            return {
+                options: {
+                    style: {
+                        color: graph_options.legend.entries[i].color
+                    }
+                },
+                value: curr
+            };
+        }, []);
 
-            "fnRowCallback": function (nRow, aData) {
-                $(nRow).css("color", aData.color);
-            }
+        $('#rrd-table').footable({
+            sorting: {
+                enabled: true
+            },
+            columns: [
+                { name: "label", title: "Action" },
+                { name: "value", title: "Messages",         defaultContent: "" },
+                { name: "min",   title: "Minimum, " + unit, defaultContent: "" },
+                { name: "avg",   title: "Average, " + unit, defaultContent: "" },
+                { name: "max",   title: "Maximum, " + unit, defaultContent: "" },
+                { name: "last",  title: "Last, " + unit },
+            ],
+            rows: rows
         });
     }
 
@@ -163,7 +176,9 @@ function($, D3Evolution, unused) {
                 scaleFactor = 60;
                 unit = "msg/min";
                 data.forEach(function (s) {
-                    s.forEach(function (d) { d.y *= scaleFactor; });
+                    s.forEach(function (d) {
+                        if (d.y !== null) { d.y *= scaleFactor; }
+                    });
                 });
             }
 
@@ -194,7 +209,7 @@ function($, D3Evolution, unused) {
             rspamd.queryNeighbours("graph", function (req_data) {
                 var neighbours_data = req_data
                     .filter(function (d) { return d.status }) // filter out unavailable neighbours
-                    .map(function (d){ return d.data; })
+                    .map(function (d){ return d.data; });
 
                 if (neighbours_data.length > 1) {
                     neighbours_data.reduce(function (res, curr) {
