@@ -2,36 +2,35 @@ requirejs.config({
     baseUrl: 'js/lib',
     paths: {
         app: '../app',
-        jquery: 'jquery-3.1.1.min',
+        jquery: 'jquery-3.2.1.min',
         visibility: 'visibility.min',
         humanize: 'humanize.min',
         bootstrap: 'bootstrap.min',
         d3: 'd3.min',
         d3evolution: 'd3evolution.min',
         d3pie: 'd3pie.min',
-        datatables: 'datatables.min',
+        footable: 'footable.min',
         bootstrap: 'bootstrap.min',
     },
     shim: {
         d3: {exports: 'd3'},
         bootstrap: {exports: 'bootstrap', deps: ['jquery']},
-        d3pie: {exports: 'd3pie', deps: ['d3', 'jquery']},
+        d3pie: {exports: 'd3pie', deps: ['d3.global', 'jquery']},
         d3evolution: {exports: 'D3Evolution', deps: ['d3', 'd3pie', 'jquery']},
-        datatables: {deps: ['bootstrap', 'jquery']},
-    },
-     map: {
-      '*': {
-        'datatables.net': 'datatables',
-      }
+        footable: {deps: ['bootstrap', 'jquery']}
     }
 });
 
+define("d3.global", ["d3"], function(_) {
+  d3 = _;
+});
+
 // Load main UI
 require(['domReady'],
 function(domReady) {
     domReady(function () {
-        require(['jquery', 'app/rspamd'],
-            function ($, rspamd) {
+        require(['jquery', 'd3', 'app/rspamd'],
+            function ($, d3, rspamd) {
                 rspamd.setup();
                 rspamd.connect();
             });
